import path from "path";
import { fs, isServiceWorker, ROUTE_SPEC_FILE, SERVER_DIR } from "@modern-js/utils";
import { MANIFEST_FILE, PKG_FILE, SERVER_HOOKS, WEB_APP_NAME, WORKER_SERVER, WORKER_SERVER_ENTRY, WRANGLER_FILE } from "./constants";
import { worker } from "./code";
const workerPlugin = () => ({
  name: "@modern-js/plugin-worker",
  setup: (ctx) => {
    return {
      async config() {
        return {
          output: {
            disableNodePolyfill: false
          }
        };
      },
      async afterDev() {
        var _configContext_deploy_worker;
        const { appDirectory, distDirectory } = ctx.useAppContext();
        const configContext = ctx.useResolvedConfigContext();
        if (!isServiceWorker(configContext)) {
          return;
        }
        const enableWorkerInWeb = configContext.deploy.enableWorker || ((_configContext_deploy_worker = configContext.deploy.worker) === null || _configContext_deploy_worker === void 0 ? void 0 : _configContext_deploy_worker.web);
        writeWorkerServerFile(appDirectory, distDirectory, enableWorkerInWeb);
      },
      async afterBuild() {
        var _configContext_deploy_worker;
        const { appDirectory, distDirectory } = ctx.useAppContext();
        const configContext = ctx.useResolvedConfigContext();
        if (!isServiceWorker(configContext)) {
          return;
        }
        const enableWorkerInWeb = configContext.deploy.enableWorker || ((_configContext_deploy_worker = configContext.deploy.worker) === null || _configContext_deploy_worker === void 0 ? void 0 : _configContext_deploy_worker.web);
        writeWorkerServerFile(appDirectory, distDirectory, enableWorkerInWeb);
      }
    };
  }
});
const writeWorkerServerFile = (appDirectory, distDirectory, enableWorkerInWeb) => {
  const workServerDir = path.join(distDirectory, WORKER_SERVER);
  const serverHookDir = path.join(appDirectory, SERVER_DIR, WEB_APP_NAME);
  const isExistsServerHook = fs.existsSync(`${serverHookDir}.ts`) || fs.existsSync(`${serverHookDir}.js`);
  const relativePath = path.relative(workServerDir, serverHookDir);
  fs.removeSync(workServerDir);
  fs.mkdirSync(workServerDir);
  fs.writeFileSync(path.join(workServerDir, WORKER_SERVER_ENTRY), worker());
  const routeJSON = path.join(distDirectory, ROUTE_SPEC_FILE);
  const { routes } = fs.readJSONSync(routeJSON);
  let importStr = ``;
  let pageStr = ``;
  const routeArr = [];
  if (isExistsServerHook && !enableWorkerInWeb) {
    importStr += `import * as ${SERVER_HOOKS} from '${relativePath}'
`;
  }
  routes.forEach((route) => {
    if (route.isSSR) {
      importStr += `import { serverRender as ${route.entryName}ServerRender } from "../${route.worker}";
`;
    }
    if (!route.isApi) {
      importStr += `import ${route.entryName}template from "../${route.entryPath}";
`;
      pageStr += `"${route.urlPath}": {
        ${isExistsServerHook && !enableWorkerInWeb ? `${SERVER_HOOKS},` : ""}
        entryName: "${route.entryName}",
        template: ${route.entryName}template,
        serverRender: ${route.isSSR ? `${route.entryName}ServerRender` : void 0},
      },`;
      routeArr.push({
        entryName: route.entryName,
        isSSR: route.isSSR,
        urlPath: route.urlPath
      });
    }
  });
  const manifest = `
${importStr}

export const manifest = {
  pages: {
    ${pageStr}
  },
  routes: ${JSON.stringify(routeArr, null, "   ")}
}
        `;
  fs.writeFileSync(path.join(workServerDir, MANIFEST_FILE), manifest);
  const pkg = fs.readJSONSync(path.join(appDirectory, PKG_FILE));
  fs.writeFileSync(path.join(distDirectory, WRANGLER_FILE), `name = "${pkg.name}"
main = "${path.join(WORKER_SERVER, WORKER_SERVER_ENTRY)}"
compatibility_date = "${(/* @__PURE__ */ new Date()).toISOString().substring(0, 10)}"
        `);
};
var src_default = workerPlugin;
export {
  src_default as default,
  workerPlugin
};
